/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *  and Art Vodolazsky <burivuh@crew.org.ru>
 *
 *  NOTE: This is init file of X Neural Switcher
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "xconfig.h"
#include "xProgram.h"
#include "utils.h"

#define CNF_NAME  "xneur/.xneurrc"
#define RUDICT_NAME "xneur/.rudict"
#define ENDICT_NAME "xneur/.endict"
#define MIN_LEN_TO_WORK 2

int main (int argc, char *argv[])
{
	bool have_client = TRUE, to_continue;
	app_list *supported_app_list = NULL;
	size word_len = 0;

	char *config_file = (char *) _malloc (strlen (getenv ("HOME")) + strlen (CNF_NAME) + 1);
	char *rudict_file = (char *) _malloc (strlen (getenv ("HOME")) + strlen (RUDICT_NAME) + 1);
	char *endict_file = (char *) _malloc (strlen (getenv ("HOME")) + strlen (ENDICT_NAME) + 1);

	xProgramInfo *xinfo = NULL;	// operation data structures
	
	xconf *xnc = NULL;	// config structure

	sprintf (config_file, "%s/%s", getenv ("HOME"), CNF_NAME);
	sprintf (rudict_file, "%s/%s", getenv ("HOME"), RUDICT_NAME);
	sprintf (endict_file, "%s/%s", getenv ("HOME"), ENDICT_NAME); 

	// load config structure
	xnc = XConfigInit (xnc);
	xnc = XConfigLoad (xnc, config_file);
	xnc->rudict = XDictionaryLoad (xnc->rudict, rudict_file);
	xnc->endict = XDictionaryLoad (xnc->endict, endict_file);
	free (rudict_file);
	free (endict_file);
	free (config_file);

	//init program structures
	xinfo = xProgram_init(xnc);
	if (xinfo == NULL)
		xProgram_exit ();

	// init current mode value
	xnc->CurrentMode = xnc->DefaultMode;
	send_error (DEBUG, "Current mode is set to %d\n", &(xnc->CurrentMode));
	send_error (DEBUG, "Current language detect mode is set to %d\n", &(xnc->LangDetectMode));
	send_error (DEBUG, "Russian Dictionary complete\n", NULL);
	send_error (DEBUG, "English Dictionary complete\n", NULL);

	//init_tray_icon(xnc);

	while (!xProgram_isExiting ()) {
		// pause between iterations
		usleep (xnc->ScaningDelay);

		// refresh iteration vars
		supported_app_list = xnc->alist;

		// update current xclient info
		have_client = xProgram_update (xinfo, xnc->alist);
		if (have_client == FALSE)
			continue;

		// so client has met our conditions. Let's parse input keys
		word_len = xProgram_processInput (xinfo);

		// perform bind actions if any
		to_continue = xProgram_performCurrentBind (xinfo, xnc);
		if (to_continue)
			continue;

		// so we have no bind during this iteration. Perform mode actions
		// if any.
                if (word_len >= MIN_LEN_TO_WORK) {
			xProgram_performCurrentMode (xinfo, xnc);}
	}

	xnc = XConfigFree (xnc);
	xProgram_free (xinfo);

	return 0;
}
